# coding: utf-8

"""
    Wink Payment API

    A programmatic way to create bookings, receive reactive and disburse funds globally.  ## APIs Not every integrator needs every APIs. For that reason, we have separated APIs into context.  - [Affiliate](/affiliate): All APIs related to selling travel inventory as an affiliate. - [Analytics](/analytics): All APIs related to tracking metrics across a wide variety of data source segments including, more entertaining, leaderboard metrics. - [Booking](/booking): All APIs related to creating platform bookings. - [Channel Manager](/channel-manager): All APIs related to channel managers who want to integrate with our platform. - [Extranet](/extranet): All APIs related to managing travel inventory and suppliers. - [Inventory](/inventory): All APIs related to retrieve known travel inventory as it was found using the Lookup API.. - [Lookup](/lookup): All APIs related to locating inventory by region, locale and property flags. - [Reference](/reference): All APIs related to retrieving platform-supported taxonomies. - [TripPay](/reactive): All APIs related to TripPay account management, booking, mapping and integration features.  ## SDKs We are actively working on supporting the most used languages out there. If you don't see your language here, reach out to us with a request to officially add your language. In the meantime, if you want to roll your own SDK, you can do so by downloading the OpenAPI spec and using one of the many available OpenAPI generators available: [https://openapi-generator.tech/docs/generators](https://openapi-generator.tech/docs/generators).  - Java SDK [https://github.com/wink-travel/trip-pay-sdk-java](https://github.com/wink-travel/trip-pay-sdk-java)  # Usage These features are made available to you via a [REST API](https://en.wikipedia.org/wiki/Representational_state_transfer). This API is language agnostic. We will link to SDKs for the most popular programming languages on this page as they become available.  ## Versioning We chose to version our endpoints in a way that we hope affects your integration with us the least. You request the version of our API you wish to work with via the `Wink-Version` header. When it's time for you to upgrade, you only have to change the version number to get access to our updated endpoints.  ## Release history - Follow updates on Github: https://github.com/wink-travel/wink-sdk-java/blob/master/CHANGELOG.md 

    The version of the OpenAPI document: 30.15.2
    Contact: bjorn@wink.travel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from trip_pay_payment.models.custom_monetary_amount import CustomMonetaryAmount
from trip_pay_payment.models.payout_fee import PayoutFee
from trip_pay_payment.models.quote import Quote
from typing import Optional, Set
from typing_extensions import Self

class Payout(BaseModel):
    """
    Payout keeps track of all outgoing funds for a certain account.
    """ # noqa: E501
    vendor: StrictStr = Field(description="Name of integration vendor")
    vendor_identifier: StrictStr = Field(description="Which acquirer account we return fund from.", alias="vendorIdentifier")
    vendor_name: StrictStr = Field(description="Name of regional acquirer account.", alias="vendorName")
    ledger_identifier: StrictStr = Field(description="Unique system ID.", alias="ledgerIdentifier")
    beneficiary_identifier: StrictStr = Field(description="Beneficiary ID.", alias="beneficiaryIdentifier")
    external_payee_identifier: StrictStr = Field(description="This would be the Wise recipient ID.", alias="externalPayeeIdentifier")
    type: StrictStr = Field(description="Type of withdrawal.")
    entry: CustomMonetaryAmount
    fees: Optional[List[PayoutFee]] = Field(default=None, description="Fees incurred when making the withdrawal.")
    quote: Optional[Quote] = None
    created: datetime = Field(description="When the payout record was created.")
    description: Optional[StrictStr] = Field(default=None, description="Textual response from provider")
    payout_id: Optional[StrictStr] = Field(default=None, description="The ledgerIdentifier that was generated when scheduling the payout. This will come from the payout provider such as Stripe.", alias="payoutId")
    reference_code: Optional[StrictStr] = Field(default=None, description="The transaction code that was generated when the funds move out of TripPay's account. This will come from the payout provider such as Stripe. E.g. For VCCs, it will occur when an authorization takes place.", alias="referenceCode")
    reference_code_date: Optional[datetime] = Field(default=None, description="The time the funds were withdrawn", alias="referenceCodeDate")
    status: StrictStr = Field(description="Status of withdrawal.")
    __properties: ClassVar[List[str]] = ["vendor", "vendorIdentifier", "vendorName", "ledgerIdentifier", "beneficiaryIdentifier", "externalPayeeIdentifier", "type", "entry", "fees", "quote", "created", "description", "payoutId", "referenceCode", "referenceCodeDate", "status"]

    @field_validator('vendor')
    def vendor_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['STRIPE', 'AGENT', 'NMI', 'WISE']):
            raise ValueError("must be one of enum values ('STRIPE', 'AGENT', 'NMI', 'WISE')")
        return value

    @field_validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['BANK_TRANSFER']):
            raise ValueError("must be one of enum values ('BANK_TRANSFER')")
        return value

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['INITIATED', 'ACTIVE', 'INACTIVE', 'FAILED', 'CANCELLED', 'COMPLETE']):
            raise ValueError("must be one of enum values ('INITIATED', 'ACTIVE', 'INACTIVE', 'FAILED', 'CANCELLED', 'COMPLETE')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Payout from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of entry
        if self.entry:
            _dict['entry'] = self.entry.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in fees (list)
        _items = []
        if self.fees:
            for _item_fees in self.fees:
                if _item_fees:
                    _items.append(_item_fees.to_dict())
            _dict['fees'] = _items
        # override the default output from pydantic by calling `to_dict()` of quote
        if self.quote:
            _dict['quote'] = self.quote.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Payout from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "vendor": obj.get("vendor"),
            "vendorIdentifier": obj.get("vendorIdentifier"),
            "vendorName": obj.get("vendorName"),
            "ledgerIdentifier": obj.get("ledgerIdentifier"),
            "beneficiaryIdentifier": obj.get("beneficiaryIdentifier"),
            "externalPayeeIdentifier": obj.get("externalPayeeIdentifier"),
            "type": obj.get("type"),
            "entry": CustomMonetaryAmount.from_dict(obj["entry"]) if obj.get("entry") is not None else None,
            "fees": [PayoutFee.from_dict(_item) for _item in obj["fees"]] if obj.get("fees") is not None else None,
            "quote": Quote.from_dict(obj["quote"]) if obj.get("quote") is not None else None,
            "created": obj.get("created"),
            "description": obj.get("description"),
            "payoutId": obj.get("payoutId"),
            "referenceCode": obj.get("referenceCode"),
            "referenceCodeDate": obj.get("referenceCodeDate"),
            "status": obj.get("status")
        })
        return _obj


